<?php
// Exit if accessed directly
if (!defined('ABSPATH'))
    exit;

/**
 * Public Pages Class
 * 
 * Handles all the different features and functions
 * for the front end pages.
 * 
 * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 * @since 1.0.0
 */
class WOO_Vou_Dot_Public {


    /**
     * Handle to calculate discount and modify cart total
     * 
     * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
     * @since 1.0.0
     */
    public function woo_vou_dot_custom_cart_total( $total, $cart) {

        global $woocommerce,$woo_vou_cart_total,$woo_vou_tax_discount, $applied_coupon_amount;

        $woo_vou_cart_total = $total;

        $prefix = WOO_VOU_META_PREFIX;
        $coupons = $woocommerce->cart->get_applied_coupons();

        if( empty( $coupons ) ){
            return $total;
        }

        $tax_inclusive = get_option('woocommerce_prices_include_tax');

        foreach ( $coupons as $key => $coupon ) {

            // arguments for getting coupon id
            $args = array(
                'fields' => 'ids',
                'name' => strtolower($coupon),
                array(
                    'key' => $prefix . 'coupon_type',
                    'value' => 'voucher_code'
                ),
            );

            // Get Coupon code data
            $coupon_code_data = woo_vou_get_coupon_details($args);

            if (!empty($coupon_code_data)) { // check if the coupon is voucher coupon 
                
                $prefix = WOO_VOU_META_PREFIX;
                $apply_discount_on = get_post_meta($coupon_code_data[0], $prefix.'discount_on_tax_type', true);

                if( $apply_discount_on == 'total' ) { // if apply discount on option is total

                    $couponObj = new WC_Coupon($coupon_code_data[0]);
                    $coupon_amount = $couponObj->get_amount();
                    $discount_tax_inclusive = false;

                    if( $tax_inclusive == 'no'){
                        $discount_tax_inclusive = true;
                    }

                    $amount = $cart->get_coupon_discount_amount( $coupon, $discount_tax_inclusive); // applied amount
                    $discoun_tax_amount = $cart->get_coupon_discount_tax_amount($coupon);
                    
                    // Store for later use
                    $applied_coupon_amount[strtolower($coupon)]['amount'] = $amount;
                    $applied_coupon_amount[strtolower($coupon)]['discoun_tax_amount'] = $discoun_tax_amount;


                    $remain_coupon_amount = $coupon_amount - $amount;

                    $fee_total = $cart->get_fee_total();
                    $fee_total_tax = $cart->get_fee_tax();

                    $tax_total  = $cart->get_taxes_total();

                    if( $tax_inclusive == 'no' && $tax_total > 0 && $total > 0 && $remain_coupon_amount > 0 ){
                        
                        if( $tax_total > $remain_coupon_amount ) { // deduct shipping charge from coupon amout

                            $total = $total - $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                        } elseif( $remain_coupon_amount > 0 && $total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $tax_total;
                            $total = $total - $tax_total;
                        }

                    }

                    // check if the shipping charge is available
                    if( $total > 0 && $cart->get_shipping_total() && $remain_coupon_amount > 0 ){

                        $shipping_charge = $cart->get_shipping_total();
                        $shipping_tax = $cart->get_shipping_tax();


                        if( $shipping_charge > $remain_coupon_amount ) { // deduct shipping charge from coupon amout

                            $total = $total - $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                        } elseif( $remain_coupon_amount > 0 && $total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $shipping_charge;
                            $total = $total - $shipping_charge;
                        }

                        // deduct shipping tax from coupon amout
                        if( $shipping_tax > $remain_coupon_amount && $remain_coupon_amount > 0 && $total > 0 ) {
                            $total = $total - $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                            $woo_vou_tax_discount = $remain_coupon_amount;
                        } elseif( $remain_coupon_amount > 0 && $total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $shipping_tax;
                            $total = $total - $shipping_tax;
                            $woo_vou_tax_discount = $woo_vou_tax_discount + $shipping_tax;
                        }

                        // deduct fees charge from coupon amout
                        if( $fee_total > $remain_coupon_amount && $remain_coupon_amount > 0 && $total > 0 ) {
                            $total = $total - $fee_total;
                            $remain_coupon_amount = 0;
                        } elseif( $remain_coupon_amount > 0 && $total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $fee_total;
                            $total = $total - $fee_total;
                        }

                    }

                    // deduct fees tax from coupon amout
                    if( $total > 0 && $fee_total > 0 && $remain_coupon_amount > 0 ){

                        if( $fee_total_tax > $remain_coupon_amount && $remain_coupon_amount > 0 ) {
                            $total = $total - $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                            $woo_vou_tax_discount = $woo_vou_tax_discount + $remain_coupon_amount;
                        } elseif( $remain_coupon_amount > 0 && $total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $fee_total_tax;
                            $total = $total - $fee_total_tax;
                            $woo_vou_tax_discount = $woo_vou_tax_discount + $fee_total_tax;
                        }
                    }

                }
            }

        }

        return $total;
    }



    /**
     * Handle to calculate discount and modify cart discount total
     * 
     * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
     * @since 1.0.0
     */
    public function woo_vou_dot_woocommerce_filter_checkout_for_coupons( $total , $cart ) { 
        
        global $woocommerce, $woo_vou_cart_total, $applied_coupon_amount;

        $prefix = WOO_VOU_META_PREFIX;
        $coupons = $woocommerce->cart->get_applied_coupons();

        if( empty( $coupons ) ){
            return $total;
        }

        $coupon_discount_totals = array();
        $coupon_discount_totals_tax = array();
        $tax_inclusive = get_option('woocommerce_prices_include_tax');

        foreach ( $coupons as $key => $coupon ) {

            // arguments for getting coupon id
            $args = array(
                'fields' => 'ids',
                'name' => strtolower($coupon),
                array(
                    'key' => $prefix . 'coupon_type',
                    'value' => 'voucher_code'
                ),
            );

            // Get Coupon code data
            $coupon_code_data = woo_vou_get_coupon_details($args);

            if ( !empty($coupon_code_data) ) {

                $prefix = WOO_VOU_META_PREFIX;
                $apply_discount_on = get_post_meta($coupon_code_data[0], $prefix.'discount_on_tax_type', true);

                if( WC()->cart->has_discount( $coupon ) && $apply_discount_on == 'total' ) { 

                    $couponObj = new WC_Coupon($coupon);

                    $coupon_amount = $couponObj->get_amount();
                    $discount_tax_inclusive = false;

                    if( $tax_inclusive == 'no'){
                        $discount_tax_inclusive = true;
                    }

                    //$amount = $cart->get_coupon_discount_amount( $coupon, $discount_tax_inclusive); // applied amount
                    //$discoun_tax_amount = $cart->get_coupon_discount_tax_amount($coupon);
                    $amount = $applied_coupon_amount[strtolower($coupon)]['amount'];
                    $discoun_tax_amount = $applied_coupon_amount[strtolower($coupon)]['discoun_tax_amount'];

                    $fee_total = $cart->get_fee_total();
                    $fee_total_tax = $cart->get_fee_tax();

                    $fee_total = !empty( $fee_total ) ? floatval( $fee_total ) : 0;
                    $fee_total_tax = !empty( $fee_total_tax ) ? floatval( $fee_total_tax ) : 0;

                    $remain_coupon_amount = $coupon_amount - $amount;
                    $temp_total = $woo_vou_cart_total;

                    $tax_total  = $cart->get_taxes_total();

                    if( $tax_inclusive == 'no' && $tax_total > 0 && $temp_total > 0 && $remain_coupon_amount >= 0 ){
                        if( $tax_total > $remain_coupon_amount ) {
                            $temp_total = $temp_total - $remain_coupon_amount;
                            $amount = $amount + $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                        } 
                        elseif( $remain_coupon_amount > 0 && $temp_total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $tax_total;
                            $amount = $amount + $tax_total;
                            $temp_total = $temp_total - $tax_total;
                        }

                         // Set coupon amout based on calculation
                        $coupon_discount_totals[$coupon] = $amount;
                        $coupon_discount_totals_tax[$coupon] = 0;
                        $cart->set_discount_total($amount);

                        $cart->set_coupon_discount_totals($coupon_discount_totals);
                        $cart->set_coupon_discount_tax_totals( wc_remove_number_precision_deep( $coupon_discount_totals_tax ) );
                    }

                    // deduct shipping charge from coupon amout
                    if( $temp_total > 0 && $cart->get_shipping_total() && $remain_coupon_amount > 0 ) {
                        
                        $shipping_charge = $cart->get_shipping_total();
                        $shipping_tax = $cart->get_shipping_tax();

                        // deduct shipping charge from coupon amout
                        if( $shipping_charge > $remain_coupon_amount ) {
                            $temp_total = $temp_total - $remain_coupon_amount;
                            $amount = $amount + $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                        } 
                        elseif( $remain_coupon_amount > 0 && $temp_total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $shipping_charge;
                            $amount = $amount + $shipping_charge;
                            $temp_total = $temp_total - $shipping_charge;
                        }

                        // deduct shipping tax from coupon amout
                        if( $shipping_tax > $remain_coupon_amount && $remain_coupon_amount > 0 && $temp_total > 0 ) {
                            $temp_total = $temp_total - $remain_coupon_amount;
                            $amount = $amount + $remain_coupon_amount;
                            $discounted_ship_tax = $shipping_tax - $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                        } elseif( $remain_coupon_amount > 0 && $temp_total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $shipping_tax;
                            $amount = $amount + $shipping_tax;
                            $temp_total = $temp_total - $shipping_tax;
                        }

                        // Set coupon amout based on calculation
                        $coupon_discount_totals[$coupon] = $amount;
                        $coupon_discount_totals_tax[$coupon] = 0;
                        $cart->set_discount_total($amount);

                        $cart->set_coupon_discount_totals($coupon_discount_totals);
                        $cart->set_coupon_discount_tax_totals( wc_remove_number_precision_deep( $coupon_discount_totals_tax ) );
                    }

                    if( $temp_total > 0 && $fee_total > 0 && $remain_coupon_amount > 0 ) {

                        // deduct Fees charge from coupon amout
                        if( $fee_total > $remain_coupon_amount && $remain_coupon_amount > 0 && $temp_total > 0 ) {
                            $temp_total = $temp_total - $remain_coupon_amount;
                            $amount = $amount + $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                        }
                        elseif( $remain_coupon_amount > 0 && $temp_total > 0){
                            $remain_coupon_amount = $remain_coupon_amount - $fee_total;
                            $amount = $amount + $fee_total;
                            $temp_total = $temp_total - $fee_total;
                        }

                        // deduct fees tax from coupon amout
                        if( $fee_total_tax > $remain_coupon_amount && $remain_coupon_amount > 0 && $temp_total > 0 ) {
                            $temp_total = $temp_total - $remain_coupon_amount;
                            $amount = $amount + $remain_coupon_amount;
                            $discounted_ship_tax = $fee_total_tax - $remain_coupon_amount;
                            $remain_coupon_amount = 0;
                        } elseif( $remain_coupon_amount > 0 && $temp_total > 0 ){
                            $remain_coupon_amount = $remain_coupon_amount - $fee_total_tax;
                            $amount = $amount + $fee_total_tax;
                            $temp_total = $temp_total - $fee_total_tax;
                        }

                        $coupon_discount_totals[$coupon] = $amount;
                        $coupon_discount_totals_tax[$coupon] = 0;
                        $cart->set_discount_total($amount);

                        $cart->set_coupon_discount_totals($coupon_discount_totals);
                        $cart->set_coupon_discount_tax_totals( wc_remove_number_precision_deep( $coupon_discount_totals_tax ) );

                    }

                }
            }

        }

        return $total; 
    }


    /**
     * Filter function to handle for exclusive tax to calculate tax on product price
     * Handle to fix tax counting issue with discount for exclusive tax 
     * 
     * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
     * @since 1.0.0
     */
    public function woo_vou_dot_add_exclusive_tax( $total_tax, $item, $cart_total ){
        
         global $woocommerce;
        $prefix = WOO_VOU_META_PREFIX;
        $coupons = $woocommerce->cart->get_applied_coupons();

        $tax_inclusive = get_option('woocommerce_prices_include_tax');

        if( empty( $coupons ) || $tax_inclusive == 'yes' )
            return $total_tax;

        foreach ( $coupons as $key => $coupon ) {

            // arguments for getting coupon id
            $args = array(
                'fields' => 'ids',
                'name' => strtolower($coupon),
                array(
                    'key' => $prefix . 'coupon_type',
                    'value' => 'voucher_code'
                ),
            );

            // Get Coupon code data
            $coupon_code_data = woo_vou_get_coupon_details($args);

            if (!empty($coupon_code_data)) {
                foreach ( $coupon_code_data  as $key => $coupon_id ) {
                    
                    $exclude_discount_on_tax = get_post_meta($coupon_id, $prefix.'discount_on_tax_type', true);
                    
                    // check if the discount code option is subtotal selected then customer will give all tax 
                    if( wc_tax_enabled() &&  !empty($exclude_discount_on_tax) && $exclude_discount_on_tax == 'subtotal' || $exclude_discount_on_tax == 'total' ){

                        return WC_Tax::calc_tax( $item->price, $item->tax_rates, $item->price_includes_tax );
                    }
                }
            }
        }
        
        return $total_tax;
    }



    /**
     * Handle to calculate tax discount and modify tax total on order page
     * 
     * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
     * @since 1.0.0
     */
    public function woo_vou_dot_change_formatted_order_total_for_tax( $formatted_total, $order, $tax_display, $display_refunded ){

        global $woo_vou_tax_discount;

        if( empty( $woo_vou_tax_discount ) ){
            return $formatted_total;
        }

        $formatted_total = wc_price( $order->get_total(), array( 'currency' => $order->get_currency() ) );
        $order_total     = $order->get_total();
        $total_refunded  = $order->get_total_refunded();
        $tax_string      = '';

        // Tax for inclusive prices.
        if ( wc_tax_enabled() && 'incl' === $tax_display ) {
            $tax_string_array = array();
            $tax_totals       = $order->get_tax_totals();

            if ( 'itemized' === get_option( 'woocommerce_tax_total_display' ) ) {
                foreach ( $tax_totals as $code => $tax ) {
                    $tax_amount         = ( $total_refunded && $display_refunded ) ? wc_price( WC_Tax::round( $tax->amount - $order->get_total_tax_refunded_by_rate_id( $tax->rate_id ) ), array( 'currency' => $order->get_currency() ) ) : $tax->formatted_amount;
                    $tax_amount = $tax->amount - $woo_vou_tax_discount;
                    $tax_string_array[] = sprintf( '%s %s', wc_price($tax_amount), $tax->label );
                }
            } elseif ( ! empty( $tax_totals ) ) {
                $tax_amount         = ( $total_refunded && $display_refunded ) ? $order->get_total_tax() - $order->get_total_tax_refunded() : $order->get_total_tax();
                $tax_amount = $tax_amount - $woo_vou_tax_discount;
                $tax_string_array[] = sprintf( '%s %s', wc_price( $tax_amount, array( 'currency' => $order->get_currency() ) ), WC()->countries->tax_or_vat() );
            }

            if ( ! empty( $tax_string_array ) ) {
                /* translators: %s: taxes */
                $tax_string = ' <small class="includes_tax">' . sprintf( __( '(includes %s)', 'woovoucher_dot' ), implode( ', ', $tax_string_array ) ) . '</small>';
            }
        }

        $tax_string = '';

        if ( $total_refunded && $display_refunded ) {
            $formatted_total = '<del>' . wp_strip_all_tags( $formatted_total ) . '</del> <ins>' . wc_price( $order_total - $total_refunded, array( 'currency' => $order->get_currency() ) ) . $tax_string . '</ins>';
        } else {
            $formatted_total .= $tax_string;
        }

        
        return $formatted_total;
    }



    /**
     * Handle to calculate tax discount and modify tax total on cart page
     * 
     * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
     * @since 1.0.0
     */
    public function woo_vou_dot_cart_total_html( $value ) {
        global $woo_vou_tax_discount;

        if( empty( $woo_vou_tax_discount ) ){
            return $value;
        }

        $value = '<strong>' . WC()->cart->get_total() . '</strong> ';

        // If prices are tax inclusive, show taxes here.
        if ( wc_tax_enabled() && WC()->cart->display_prices_including_tax() ) {
            $tax_string_array = array();
            $cart_tax_totals  = WC()->cart->get_tax_totals();

            if ( get_option( 'woocommerce_tax_total_display' ) === 'itemized' ) {
                foreach ( $cart_tax_totals as $code => $tax ) {
                    $tax_amount = $tax->amount;
                    $tax_amount = $tax_amount - $woo_vou_tax_discount;
                    $tax_string_array[] = sprintf( '%s %s', wc_price($tax_amount), $tax->label );
                }
            } elseif ( ! empty( $cart_tax_totals ) ) {
                $tax_amount = WC()->cart->get_taxes_total( true, true );

                $tax_amount = $tax_amount - $woo_vou_tax_discount;
                $tax_string_array[] = sprintf( '%s %s', wc_price( $tax_amount ), WC()->countries->tax_or_vat() );
            }

            if ( ! empty( $tax_string_array ) ) {
                $taxable_address = WC()->customer->get_taxable_address();
                /* translators: %s: country name */
                $estimated_text = WC()->customer->is_customer_outside_base() && ! WC()->customer->has_calculated_shipping() ? sprintf( ' ' . __( 'estimated for %s', 'woovoucher_dot' ), WC()->countries->estimated_for_prefix( $taxable_address[0] ) . WC()->countries->countries[ $taxable_address[0] ] ) : '';
                $value .= '<small class="includes_tax">('
                            /* translators: includes tax information */
                            . esc_html__( 'includes', 'woovoucher_dot' )
                            . ' '
                            . wp_kses_post( implode( ', ', $tax_string_array ) )
                            . esc_html( $estimated_text )
                            . ')</small>';
            }
        }

        return $value;
    }

	
	 /**
     * Adding Hooks
     * 
     * Adding proper hoocks for the discount codes
     * 
 	* @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 	* @since 1.0.0
     */
    public function add_hooks() {

        add_filter('woocommerce_calculated_total', array( $this, 'woo_vou_dot_custom_cart_total'),10,2);

        add_filter( 'woocommerce_calculated_total', array( $this, 'woo_vou_dot_woocommerce_filter_checkout_for_coupons'), 10, 2 );

        add_filter('woocommerce_calculate_item_totals_taxes', array( $this, 'woo_vou_dot_add_exclusive_tax' ),10,3 );

        add_filter('woocommerce_get_formatted_order_total', array( $this, 'woo_vou_dot_change_formatted_order_total_for_tax'),10,4);
    	add_filter('woocommerce_cart_totals_order_total_html', array( $this,'woo_vou_dot_cart_total_html'),10,1);


    }
}