<?php
/**
 * Plugin Name: WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 * Plugin URI:  https://wpwebelite.com/
 * Description: With WooCommerce PDF Vouchers - Redeem Voucher on Cart Total Extension, you can apply discount on cart total using voucher coupon
 * Version: 1.0.2
 * Author: WPWeb
 * Author URI: https://wpwebelite.com/
 * Text Domain: woovoucher_dot
 * Domain Path: languages
 * 
 * WC tested up to: 4.2.0
 * Tested up to: 5.4.1
 * 
 * @package WooCommerce - PDF Vouchers
 * @category Core
 * @author WPWeb
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Basic plugin definitions
 * 
 * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 * @since 1.0.0
 */
if( !defined( 'WOO_VOU_DOT_PLUGIN_VERSION' ) ) {
	define( 'WOO_VOU_DOT_PLUGIN_VERSION', '1.0.2' ); //Plugin version number
}
if( !defined( 'WOO_VOU_DOT_DIR' ) ) {
	define( 'WOO_VOU_DOT_DIR', dirname( __FILE__ ) ); // plugin dir
}
if( !defined( 'WOO_VOU_DOT_URL' ) ) {
	define( 'WOO_VOU_DOT_URL', plugin_dir_url( __FILE__ ) ); // plugin url
}
if( !defined( 'WOO_VOU_DOT_ADMIN' ) ) {
	define( 'WOO_VOU_DOT_ADMIN', WOO_VOU_DOT_DIR . '/includes/admin' ); // plugin admin dir
}
if( !defined( 'WOO_VOU_DOT_IMG_DIR' ) ) {
	define( 'WOO_VOU_DOT_IMG_DIR', WOO_VOU_DOT_DIR.'/includes/images' ); // plugin image dir
}
if( !defined( 'WOO_VOU_DOT_IMG_URL' ) ) {
	define( 'WOO_VOU_DOT_IMG_URL', WOO_VOU_DOT_URL.'includes/images' ); // plugin image url
}
if( !defined( 'WOO_VOU_DOT_PLUGIN_BASENAME' ) ) {
	define( 'WOO_VOU_DOT_PLUGIN_BASENAME', basename( WOO_VOU_DOT_DIR ) ); //Plugin base name
}
if ( ! defined( 'WOO_VOU_DOT_PLUGIN_KEY' ) ) {
	define( 'WOO_VOU_DOT_PLUGIN_KEY', 'woovoucherdot' );
}

/**
 * Load Text Domain
 * 
 * This gets the plugin ready for translation.
 * 
 * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 * @since 1.0.0
 */
function woo_vou_dot_load_text_domain() {
	
	// Set filter for plugin's languages directory
	$woo_vou_dot_lang_dir	= dirname( plugin_basename( __FILE__ ) ) . '/languages/';
	$woo_vou_dot_lang_dir	= apply_filters( 'woo_vou_dot_languages_directory', $woo_vou_dot_lang_dir );
	
	// Traditional WordPress plugin locale filter
	$locale	= apply_filters( 'plugin_locale',  get_locale(), 'woovoucher_dot' );
	$mofile	= sprintf( '%1$s-%2$s.mo', 'woovoucher_dot', $locale );
	
	// Setup paths to current locale file
	$mofile_local	= $woo_vou_dot_lang_dir . $mofile;
	$mofile_global	= WP_LANG_DIR . '/' . WOO_VOU_DOT_PLUGIN_BASENAME . '/' . $mofile;
	
	if ( file_exists( $mofile_global ) ) { // Look in global /wp-content/languages/woocommerce-pdf-vouchers folder
		load_textdomain( 'woovoucher_dot', $mofile_global );
	} elseif ( file_exists( $mofile_local ) ) { // Look in local /wp-content/plugins/woocommerce-pdf-vouchers/languages/ folder
		load_textdomain( 'woovoucher_dot', $mofile_local );
	} else { // Load the default language files
		load_plugin_textdomain( 'woovoucher_dot', false, $woo_vou_dot_lang_dir );
	}
}

//add action to load plugin
add_action( 'plugins_loaded', 'woo_vou_dot_plugin_loaded', 999);

/**
 * Check WooCommerce PDF Vouchers Plugin
 *
 * Handles to check WooCoommerce Pdf voucher plugin is enabled
 * if not activated then deactivate our plugin
 *
 * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 * @since 1.0.0
 */
function woo_vou_dot_check_activation() {

	// Check if constants are defined and PDF Vouchers plugin is active
	if ( !defined('WOO_VOU_PLUGIN_BASENAME') && !defined('WOO_VOU_PLUGIN_BASE_FILENAME') 
		&& !is_plugin_active('woocommerce-pdf-vouchers/woocommerce-pdf-vouchers.php') ) {

		// is this plugin active?
		if ( is_plugin_active( plugin_basename( __FILE__ ) ) ) {
			// deactivate the plugin
	 		deactivate_plugins( plugin_basename( __FILE__ ) );
	 		// unset activation notice
	 		unset( $_GET[ 'activate' ] );
	 		// display notice
	 		add_action( 'admin_notices', 'woo_vou_dot_admin_notices' );
		}
	}
}

//Check WooCommerce Pdf voucher plugin is Activated or not
add_action( 'admin_init', 'woo_vou_dot_check_activation' );



/**
 * Show admin notice if the main plugin is not activated
 *
 * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 * @since 1.0.0
*/
function woo_vou_dot_admin_notices() {
	
	if ( !defined( 'WOO_VOU_DIR' ) ) {
		
		echo '<div class="error">';
		echo "<p><strong>" . esc_html__( 'Woocommerce PDF Vouchers needs to be activated to be able to use the WooCommerce PDF Vouchers - Redeem Voucher on Cart Total.', 'woovoucher_dot' ) . "</strong></p>";
		echo '</div>';
	}
}

/**
 * Load Plugin
 * 
 * Handles to load plugin after
 * dependent plugin is loaded
 * successfully
 * 
 * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
 * @since 1.0.0
 */
function woo_vou_dot_plugin_loaded(){
	//check Woocommerce Pdf Voucher is activated or not
	if( defined( 'WOO_VOU_DIR' ) ) {

		/**
		 * Add plugin action links
		 *
		 * Adds a Settings, Support and Docs link to the plugin list.
		 *
		 * @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
		 * @since 1.0.0
		 */
		function woo_vou_dot_add_plugin_links( $links ) {
			$plugin_links = array(
				'<a href="'.esc_url('https://support.wpwebelite.com/').'">' . esc_html__( 'Support', 'woovoucher_dot' ) . '</a>',
				'<a href="'.esc_url('https://docs.wpwebelite.com/woocommerce-pdf-vouchers/discount-methods/#carttotal_method').'">' . esc_html__( 'Docs', 'woovoucher_dot' ) . '</a>'
			);

			return array_merge( $plugin_links, $links );
		}

		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'woo_vou_dot_add_plugin_links' );
		// load first plugin text domain
		woo_vou_dot_load_text_domain();

		global $woo_vou_dot_public, $woo_vou_dot_admin;

		//Public Class to handles most of functionalities of public side
		require_once( WOO_VOU_DOT_DIR . '/includes/class-woo-vou-redeem-on-total-public.php');
		$woo_vou_dot_public = new WOO_Vou_Dot_Public();
		$woo_vou_dot_public->add_hooks();
		
		//Admin Pages Class for admin side
		require_once( WOO_VOU_DOT_ADMIN . '/class-woo-vou-redeem-on-total-admin.php' );
		$woo_vou_dot_admin = new WOO_Vou_Dot_Admin();
		$woo_vou_dot_admin->add_hooks();
	}

	
	if( defined( 'WOO_VOU_WPWEB_UPD_DIR' ) ) {
	
		// Plugin updates
		wpweb_queue_update( plugin_basename( __FILE__ ), WOO_VOU_DOT_PLUGIN_KEY );

		include_once( WOO_VOU_WPWEB_UPD_DIR . '/updates/class-plugin-update-checker.php' );
		
		$WpwebWoovouPdfFontsUpdateChecker = new WpwebPluginUpdateChecker(
				WPWEB_UPD_DOMAIN . '/Updates/WOOVouchersDOT/info.json',
				__FILE__,
				WOO_VOU_DOT_PLUGIN_KEY
			);
	}
}



/**
* Declare compatible with HPOS
* 
* @package WooCommerce PDF Vouchers - Redeem Voucher on Cart Total
* @since 1.0.2
*/
add_action( 'before_woocommerce_init', function() {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
} );